@@ -11,8 +11,14 @@ Author: Klaus Bremer
 
 import datetime
 
+from ..core.exceptions import FritzAuthorizationError
 from ..core.fritzconnection import FritzConnection
-from . utils import get_cli_arguments, get_instance, print_header
+from .utils import (
+    get_cli_arguments,
+    get_instance,
+    print_header,
+    print_common_exception_message
+)
 
 
 class FritzInspection:
@@ -81,11 +87,13 @@ class FritzInspection:
         """
         print()
         system_info = self.fc.device_manager.system_info
-        print(
-            f"system : {system_info[-1]}\n"
-            f"build  : {system_info[-2]}\n"
-            f"hw-code: {system_info[0]}"
-        )
+        # some devices may return None as system_info
+        if system_info:
+            print(
+                f"system : {system_info[-1]}\n"
+                f"build  : {system_info[-2]}\n"
+                f"hw-code: {system_info[0]}"
+            )
         now = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
         print(f"Report date: {now}")
         for service_name, service in self.fc.services.items():
@@ -147,13 +155,19 @@ def run_inspector(inspector, args):
     print()
 
 
-def main():
-    """CLI entry point."""
+def execute():
     args = get_cli_arguments(add_arguments)
     fc = get_instance(FritzConnection, args)
     inspector = FritzInspection(fc=fc)
     run_inspector(inspector, args)
 
 
+def main():
+    try:
+        execute()
+    except FritzAuthorizationError as err:
+        print_common_exception_message(err)
+
+
 if __name__ == '__main__':
     main()
