@@ -7,12 +7,16 @@ Module to communicate with the AVM Fritz!Box.
 # Author: Klaus Bremer
 
 
+from __future__ import annotations
+
 import json
 import os
 import pickle
 import string
 import xml.etree.ElementTree as ElementTree
 from pathlib import Path
+from typing import Any
+# from typing import Union  # for python < 3.10
 
 import requests
 from requests.auth import HTTPDigestAuth
@@ -24,6 +28,7 @@ from .exceptions import (
     FritzServiceError,
 )
 from .fritzhttp import FritzHttp
+from .processor import Service  # import for typ-hint
 from .soaper import Soaper
 from .utils import (
     get_bool_env,
@@ -147,7 +152,7 @@ class FritzConnection:
     .. versionadded:: 1.6
 
     The flag `use_cache` activates caching (default `False`). Caching
-    can speed up instanciation significantly. The cached data are
+    can speed up instantiation significantly. The cached data are
     specific for the router ip, the router model and the installed
     FritzOS version. Multiple devices in the network can have separate
     cache-fies and can get used in parallel. By default the cache files
@@ -165,22 +170,29 @@ class FritzConnection:
     (json|pickle) and FRITZ_CACHEDIRECTORY (a path).
 
     .. versionadded:: 1.10
+
+    `redact_debug_log` accepts a boolean for enabling redacting sensitiv
+    data (i.e. phone numbers) in debug outputs. Default is `False`.
+
+    .. versionadded:: 1.15
+
     """
 
     def __init__(
         self,
-        address=None,
-        port=None,
-        user=None,
-        password=None,
-        timeout=None,
-        use_tls=False,
-        use_cache=False,
-        verify_cache=True,
-        cache_directory=None,
-        cache_format=None,
-        pool_connections=DEFAULT_POOL_CONNECTIONS,
-        pool_maxsize=DEFAULT_POOL_MAXSIZE,
+        address: str | None = None,
+        port: int | None = None,
+        user: str | None = None,
+        password: str | None = None,
+        timeout: float | None = None,
+        use_tls: bool = False,
+        use_cache: bool = False,
+        verify_cache: bool = True,
+        cache_directory: str | Path | None = None,
+        cache_format: str | None = None,
+        pool_connections: int = DEFAULT_POOL_CONNECTIONS,
+        pool_maxsize: int = DEFAULT_POOL_MAXSIZE,
+        redact_debug_log: bool = False
     ):
         """
         Initialisation of FritzConnection: reads all data from the box
@@ -214,7 +226,7 @@ class FritzConnection:
 
         `use_cache` is a boolean whether a cache should get used for the
         router api data. By default the api data are loaded from the
-        router at instanciation time what can take several seconds to
+        router at instantiation time what can take several seconds to
         complete. `cache_directory` is the path to the directory storing
         the cached data. By default this is a folder named
         '.fritzconnection' in the users home-directory. `cache_format`
@@ -232,6 +244,9 @@ class FritzConnection:
         `pool_connections` and `pool_maxsize` accept integers for
         changing the default urllib3 settings in order to modify the
         number of reusable connections.
+
+        `redact_debug_log` accepts a boolean for enabling redacting some
+        sensitiv data in debug outputs. Default is `False`.
         """
         if address is None:
             address = FRITZ_IP_ADDRESS
@@ -262,9 +277,11 @@ class FritzConnection:
         adapter = requests.adapters.HTTPAdapter(
             pool_connections=pool_connections, pool_maxsize=pool_maxsize)
         session.mount(PROTOCOLS[use_tls], adapter)
-        # cache for updatecheck as @functools.cached_property
-        # needs Python >= 3.8:
-        self._updatecheck = None
+
+        # TODO: remove 'self._updatecheck' when 3.7 is no longer supported.
+        # this is a dictionary-based cache
+        self._updatecheck: dict | None = None
+
         # store as instance attributes for use by library modules
         self.address = address
         self.session = session
@@ -272,7 +289,7 @@ class FritzConnection:
         self.port = port
 
         self.soaper = Soaper(
-            address, port, user, password, timeout=timeout, session=session
+            address, port, user, password, timeout=timeout, session=session, redact_debug_log=redact_debug_log
         )
         self.device_manager = DeviceManager(timeout=timeout, session=session)
         self._load_router_api(
@@ -290,29 +307,31 @@ class FritzConnection:
             f"FRITZ!OS: {self.system_version}"
         )
 
+    # TODO: change type-hint to -> dict[str, Service]
+    #  when Python 3.8 is no longer supported
     @property
-    def services(self):
+    def services(self) -> dict[str, Service]:
         """
         Dictionary of service instances. Keys are the service names.
         """
         return self.device_manager.services
 
     @property
-    def modelname(self):
+    def modelname(self) -> str:
         """
         Returns the modelname of the router.
         """
         return self.device_manager.modelname
 
     @property
-    def system_version(self):
+    def system_version(self) -> str:
         """
         Returns system version if known, otherwise None.
         """
         return self.device_manager.system_version
 
     @property
-    def device_description(self):
+    def device_description(self) -> str:
         """
         Returns a string with the device description. This is a
         combination of the device model name and the installed software
@@ -320,8 +339,11 @@ class FritzConnection:
         """
         return self.call_action("DeviceInfo1", "GetInfo")["NewDescription"]
 
+    # TODO: change to @functools.cached_property
+    # when stopping support for Python 3.7
+    # in this case the self._updatecheck instance attribute is no longer needed.
     @property
-    def updatecheck(self):
+    def updatecheck(self) -> dict:
         """
         Dictionary with information about the hard- and software version of
         the device according to "http://fritz.box/jason_boxinfo.xml".
@@ -338,7 +360,7 @@ class FritzConnection:
         return self._updatecheck
 
     @staticmethod
-    def normalize_name(name):
+    def normalize_name(name: str) -> str:
         """
         Returns the normalized service name. E.g. `WLANConfiguration` and
         `WLANConfiguration:1` will get converted to `WLANConfiguration1`.
@@ -351,7 +373,7 @@ class FritzConnection:
         return name
 
     @staticmethod
-    def set_protocol(url, use_tls):
+    def set_protocol(url: str, use_tls: bool) -> str:
         """
         Sets the protocol of the `url` according to the `use_tls`-flag
         and returns the modified `url`. Does not check whether the `url`
@@ -399,14 +421,17 @@ class FritzConnection:
     # public api:
     # -------------------------------------------
 
+    # TODO: type-hint for arguments could be
+    # `arguments: Optional[dict[str, Union[str, int, bool]]] = None`
+    # for Python >= 3.9
     def call_action(
         self,
-        service_name,
-        action_name,
+        service_name: str,
+        action_name: str,
         *,
-        arguments=None,
+        arguments: dict | None = None,
         **kwargs
-    ):
+    ) -> dict[str, Any]:
         """
         Executes the given action of the given service. Both parameters
         are required. Arguments are optional and can be provided as a
@@ -442,10 +467,10 @@ class FritzConnection:
 
     def call_http(
         self,
-        command,
-        identifier=None,
+        command: str,
+        identifier: str | None = None,
         **kwargs
-    ):
+    ) -> dict[str, str]:
         """
         Excecutes the given command by means of the http-interface. This
         can be useful for homeautomation-task currently not provided by
@@ -485,18 +510,30 @@ class FritzConnection:
             "content": content
         }
 
-    def reconnect(self):
+    def reconnect(self) -> None:
         """
         Terminate the connection and reconnects with a new ip.
         """
         self.call_action("WANIPConn1", "ForceTermination")
 
-    def reboot(self):
+    def reboot(self) -> None:
         """
         Reboot the system.
         """
         self.call_action("DeviceConfig1", "Reboot")
 
+    def get_cpu_temperatures(self) -> list[int]:
+        """
+        Returns a list of the last measured cpu-temperatures.
+        The most recent entry is the first one in the list.
+        NOTE: this function call is experimental as it is based on a
+        non-public API.
+        """
+        url = f"{self.http_interface.router_url}/query.lua"
+        payload = {"CPUTEMP": "cpu:status/StatTemperature"}
+        response = self.http_interface.call_url(url, payload)
+        return list(map(int, response.json()["CPUTEMP"].split(",")))
+
     # -------------------------------------------
     # internal methods to load router-api:
     # -------------------------------------------
@@ -552,10 +589,18 @@ class FritzConnection:
         """
         # system_id is something like ('FRITZ!Box 7590', '154.07.29') which
         # originates from the device description and is part of the cache data.
-        cached_id = (
-            self.device_manager.modelname,
-            self.device_manager.system_info[-1]
-        )
+        try:
+            cached_id = (
+                self.device_manager.modelname,
+                self.device_manager.system_info[-1]
+            )
+        except TypeError:
+            # this can happen if the default ip is used with multiple
+            # devices in the local network, so an arbitrary device gets
+            # addressed. In case the device is unable to provide
+            # system_info this value is set to `None` and a TypeError gets
+            # raised. To avoid this provide an ip as the address-argument.
+            return False
         # retrive the same information from the updatecheck property.
         # If the result is the same, the cache-data can considered as valid.
         try:
