@@ -7,8 +7,15 @@ Module to access and control the known hosts.
 # Author: Klaus Bremer
 
 
+from __future__ import annotations
+
 import itertools
 
+try:
+    from typing import Generator
+except ImportError:
+    from collections.abc import Generator
+
 from ..core.exceptions import (
     FritzActionError,
     FritzArgumentError,
@@ -36,12 +43,12 @@ class FritzHosts(AbstractLibraryBase):
         return self.fc.call_action(SERVICE, actionname, arguments=arguments, **kwargs)
 
     @property
-    def host_numbers(self):
+    def host_numbers(self) -> int:
         """The number of known hosts."""
         result = self._action("GetHostNumberOfEntries")
         return result["NewHostNumberOfEntries"]
 
-    def get_generic_host_entry(self, index):
+    def get_generic_host_entry(self, index: int) -> dict:
         """
         Returns a dictionary with information about a device internally
         registered by the position *index*. Index-positions are
@@ -49,7 +56,7 @@ class FritzHosts(AbstractLibraryBase):
         """
         return self._action("GetGenericHostEntry", NewIndex=index)
 
-    def get_generic_host_entries(self):
+    def get_generic_host_entries(self) -> Generator[dict, None, None]:
         """
         Generator returning a dictionary for every host as provided by
         `get_generic_host_entry()`. (See also `get_hosts_info()` that
@@ -61,14 +68,14 @@ class FritzHosts(AbstractLibraryBase):
             except IndexError:
                 break
 
-    def get_specific_host_entry(self, mac_address):
+    def get_specific_host_entry(self, mac_address: str) -> dict:
         """
         Returns a dictionary with information about a device addressed
         by the MAC-address.
         """
         return self._action("GetSpecificHostEntry", NewMACAddress=mac_address)
 
-    def get_specific_host_entry_by_ip(self, ip):
+    def get_specific_host_entry_by_ip(self, ip: str) -> dict:
         """
         Returns a dictionary with information about a device addressed
         by the ip-address. Provides additional information about
@@ -76,7 +83,7 @@ class FritzHosts(AbstractLibraryBase):
         """
         return self._action("X_AVM-DE_GetSpecificHostEntryByIP", NewIPAddress=ip)
 
-    def get_host_status(self, mac_address):
+    def get_host_status(self, mac_address: str) -> bool | None:
         """
         Provides status information about the device with the given
         `mac_address`. Returns `True` if the device is active or `False`
@@ -89,7 +96,7 @@ class FritzHosts(AbstractLibraryBase):
             return None
         return result["NewActive"]
 
-    def get_active_hosts(self):
+    def get_active_hosts(self) -> list[dict]:
         """
         Returns a list of dicts with information about the active
         devices. The dict-keys are: 'ip', 'name', 'mac', 'status',
@@ -97,7 +104,7 @@ class FritzHosts(AbstractLibraryBase):
         """
         return [host for host in self.get_hosts_info() if host["status"]]
 
-    def get_hosts_info(self):
+    def get_hosts_info(self) -> list[dict]:
         """
         Returns a list of dicts with information about the known hosts.
         The dict-keys are: 'ip', 'name', 'mac', 'status',
@@ -123,7 +130,7 @@ class FritzHosts(AbstractLibraryBase):
             )
         return result
 
-    def get_mesh_topology(self, raw=False):
+    def get_mesh_topology(self, raw=False) -> dict | str:
         """
         Returns information about the mesh network topology. If `raw` is
         `False` the topology gets returned as a dictionary with a list
@@ -139,7 +146,7 @@ class FritzHosts(AbstractLibraryBase):
                 raise FritzActionError(message)
             return response.text if raw else response.json()
 
-    def get_wakeonlan_status(self, mac_address):
+    def get_wakeonlan_status(self, mac_address: str) -> bool:
         """
         Returns a boolean whether wake on LAN signal gets send to the
         device with the given `mac_address` in case of a remote access.
@@ -149,7 +156,7 @@ class FritzHosts(AbstractLibraryBase):
         )
         return info["NewAutoWOLEnabled"]
 
-    def set_wakeonlan_status(self, mac_address, status=False):
+    def set_wakeonlan_status(self, mac_address: str, status: bool = False) -> None:
         """
         Sets whether a wake on LAN signal should get send send to the
         device with the given `mac_address` in case of a remote access.
@@ -162,7 +169,7 @@ class FritzHosts(AbstractLibraryBase):
         }
         self._action("X_AVM-DE_SetAutoWakeOnLANByMACAddress", arguments=args)
 
-    def set_host_name(self, mac_address, name):
+    def set_host_name(self, mac_address: str, name: str) -> None:
         """
         Sets the hostname of the device with the given `mac_address` to
         the new `name`.
@@ -173,14 +180,14 @@ class FritzHosts(AbstractLibraryBase):
         }
         self._action("X_AVM-DE_SetHostNameByMACAddress", arguments=args)
 
-    def get_host_name(self, mac_address):
+    def get_host_name(self, mac_address: str) -> str:
         """
         Returns a String with the host_name of the device with the given
         mac_address
         """
         return self.get_specific_host_entry(mac_address)["NewHostName"]
 
-    def run_host_update(self, mac_address):
+    def run_host_update(self, mac_address: str) -> None:
         """
         Triggers the host with the given `mac_address` to run a system
         update. The method returns immediately, but for the device it
@@ -190,7 +197,7 @@ class FritzHosts(AbstractLibraryBase):
         """
         self._action("X_AVM-DE_HostDoUpdate", NewMACAddress=mac_address)
 
-    def get_hosts_attributes(self):
+    def get_hosts_attributes(self) -> list[dict]:
         """
         Returns a list of dictionaries with information about all hosts.
 
@@ -204,6 +211,7 @@ class FritzHosts(AbstractLibraryBase):
         .. versionadded:: 1.10
         """
         result = self._action("X_AVM-DE_GetHostListPath")
-        url = result["NewX_AVM-DE_HostListPath"]
-        storage = HostStorage(get_xml_root(url, self.fc.session))
+        path = result["NewX_AVM-DE_HostListPath"]
+        url = f"{self.fc.address}:{self.fc.port}{path}"
+        storage = HostStorage(get_xml_root(source=url, session=self.fc.session))
         return storage.hosts_attributes
