@@ -6,13 +6,14 @@ Module to get information about WLAN devices.
 # License: MIT (https://opensource.org/licenses/MIT)
 # Author: Bernd Strebel, Klaus Bremer
 
+
+from __future__ import annotations
+
 import io
 import itertools
 import random
 import string
 
-from warnings import warn
-
 from ..core.exceptions import FritzServiceError
 from .fritzbase import AbstractLibraryBase
 
@@ -26,42 +27,28 @@ else:
 # important: don't set an extension number here:
 SERVICE = 'WLANConfiguration'
 DEFAULT_PASSWORD_LENGTH = 12
-WPA_SECURITY = 'WPA'
-NO_PASS = 'nopass'
+_QR_SECURITY_WPA = 'WPA'
+_QR_SECURITY_WEP  = 'WEP'
+_QR_SECURITY_NO_PASS = 'nopass'
+_BEACONTYPE_TO_QR_SECURITY = {
+    'WPA': _QR_SECURITY_WPA,
+    '11i': _QR_SECURITY_WPA,
+    'WPAand11i': _QR_SECURITY_WPA,
+    'WPA3': _QR_SECURITY_WPA,
+    '11iandWPA3': _QR_SECURITY_WPA,
+    'Basic': _QR_SECURITY_WEP,
+    'OWE': _QR_SECURITY_NO_PASS,
+    'OWETrans': _QR_SECURITY_NO_PASS,
+    'None': _QR_SECURITY_NO_PASS,
+}
 
 
-def get_beacon_security(instance, security):
-    """
-    Returns the beacon-security as a string based on the security
-    argument. Possible return values are 'nopass' and 'WPA'. If the
-    security is None or an empty string, the function tries to find the
-    proper security setting ('nopass'|'WPA'). If security is neither
-    None nor an empty string, the value is returned as is.
-
-    This function is not intended to get called directly.
-
-    .. versionadded:: 1.10
-    """
-    if not security:
-        info = instance.get_info()
-        beacontypes = set(info["NewX_AVM-DE_PossibleBeaconTypes"].split(","))
-        beacontypes -= set(('None', 'OWETrans'))
-        beacontype = info["NewBeaconType"]
-        if beacontype in beacontypes:
-            security = WPA_SECURITY
-        else:
-            security = NO_PASS
-    return security
-
-
-def get_wifi_qr_code(instance, kind='svg',
-                     security=None, hidden=False,
-                     scale=4):
+def _get_wifi_qr_code(instance, kind='svg', security=None, scale=4, border=0):
     """
     Returns a file-like object providing a bytestring representing a
     qr-code for wlan access. `instance` is a FritzWLAN or FritzGuestWLAN
     instance. `kind` describes the type of the qr-code. Supported types
-    are: 'svg', 'png' and 'pdf'. Default is 'svg'.
+    are: 'svg', 'png', 'pdf', 'text' and 'text-compact'. Default is 'svg'.
 
     This function is not intended to get called directly. Instead it is
     available as a method on FritzWLAN instances (as well as on
@@ -81,6 +68,11 @@ def get_wifi_qr_code(instance, kind='svg',
         with open('qr_code.png', 'wb') as fobj:
             fobj.write(stream.read())
 
+    The stream can be printed out if qr-code is represented as text: ::
+
+        stream = guest_wlan.get_wifi_qr_code(kind='text')
+        print(stream.getvalue())
+
     If `segno` is not installed the call will trigger an
     AttributeError when called on an instance and a NameError when
     called directly.
@@ -103,15 +95,24 @@ def get_wifi_qr_code(instance, kind='svg',
     .. versionadded:: 1.10
 
     """
-    stream = io.BytesIO()
-    security = get_beacon_security(instance, security)
+    if not security:
+        # if beacon type is something unknown, assume a "no pass" connection:
+        security = _BEACONTYPE_TO_QR_SECURITY.get(instance.beacontype,
+                                                  _QR_SECURITY_NO_PASS)
+    password = instance.get_password() if security != _QR_SECURITY_NO_PASS else None
     qr_code = segno.helpers.make_wifi(
         ssid=instance.ssid,
-        password=instance.get_password(),
+        password=password,
         security=security,
-        hidden=hidden
+        hidden=instance.is_hidden
     )
-    qr_code.save(out=stream, kind=kind, scale=scale)
+    if kind in ['text', 'text-compact']:
+        stream = io.StringIO()
+        compact = kind != 'text'
+        qr_code.terminal(out=stream, border=border, compact=compact)
+    else:
+        stream = io.BytesIO()
+        qr_code.save(out=stream, kind=kind, scale=scale, border=border)
     stream.seek(0)
     return stream
 
@@ -119,7 +120,7 @@ def get_wifi_qr_code(instance, kind='svg',
 def _qr_code_enabler(cls):
     """Classdecorator to inject qr-capabilities at import time."""
     if SEGNO_INSTALLED:
-        cls.get_wifi_qr_code = get_wifi_qr_code
+        cls.get_wifi_qr_code = _get_wifi_qr_code
     return cls
 
 
@@ -145,7 +146,7 @@ class FritzWLAN(AbstractLibraryBase):
         return self.fc.call_action(service, actionname, **kwargs)
 
     @property
-    def host_number(self):
+    def host_number(self) -> int:
         """
         Number of registered wlan devices for the active
         WLANConfiguration.
@@ -154,7 +155,7 @@ class FritzWLAN(AbstractLibraryBase):
         return result['NewTotalAssociations']
 
     @property
-    def total_host_number(self):
+    def total_host_number(self) -> int:
         """
         Total NewAssociatedDeviceIndexNumber of registered wlan devices
         for all WLANConfigurations.
@@ -171,17 +172,17 @@ class FritzWLAN(AbstractLibraryBase):
         return total
 
     @property
-    def ssid(self):
+    def ssid(self) -> str:
         """The WLAN SSID"""
         result = self._action('GetSSID')
         return result['NewSSID']
 
     @ssid.setter
-    def ssid(self, value):
+    def ssid(self, value: str) -> None:
         self._action('SetSSID', NewSSID=value)
 
     @property
-    def beacontype(self):
+    def beacontype(self) -> str:
         """
         Represents the beacontype for the network as string.
         At time of writing (OS 7.29) possible values are:
@@ -192,24 +193,21 @@ class FritzWLAN(AbstractLibraryBase):
         return self.get_info()['NewBeaconType']
 
     @property
-    def channel(self):
+    def is_hidden(self) -> bool:
+        """Returns True if the SSID hidden (not advertised through beacons)."""
+        return not self._action('GetBeaconAdvertisement')['NewBeaconAdvertisementEnabled']
+
+    @property
+    def channel(self) -> int:
         """The WLAN channel in use"""
         return self.channel_info()['NewChannel']
 
     @property
-    def alternative_channels(self):
+    def alternative_channels(self) -> str:
         """Alternative channels (as string)"""
         return self.channel_info()['NewPossibleChannels']
 
-    def channel_infos(self):
-        """
-        .. deprecated:: 1.9.0
-           Use :func:`channel_info` instead.
-        """
-        warn('This method is deprecated. Use "channel_info" instead.', DeprecationWarning)
-        return self.channel_info()
-
-    def channel_info(self):
+    def channel_info(self) -> dict:
         """
         Return a dictionary with the keys *NewChannel* and
         *NewPossibleChannels* indicating the active channel and
@@ -217,14 +215,14 @@ class FritzWLAN(AbstractLibraryBase):
         """
         return self._action('GetChannelInfo')
 
-    def set_channel(self, number):
+    def set_channel(self, number: int) -> None:
         """
         Set a new channel. *number* must be a valid channel number for
         the active WLAN. (Valid numbers are listed by *alternative_channels*.)
         """
         self._action('SetChannel', NewChannel=number)
 
-    def get_generic_host_entry(self, index):
+    def get_generic_host_entry(self, index: int) -> dict:
         """
         Return a dictionary with information about the device
         internally stored at the position 'index'.
@@ -235,7 +233,7 @@ class FritzWLAN(AbstractLibraryBase):
         )
         return result
 
-    def get_specific_host_entry(self, mac_address):
+    def get_specific_host_entry(self, mac_address: str) -> dict:
         """
         Return a dictionary with information about the device
         with the given 'mac_address'.
@@ -246,7 +244,7 @@ class FritzWLAN(AbstractLibraryBase):
         )
         return result
 
-    def get_hosts_info(self):
+    def get_hosts_info(self) -> list[dict]:
         """
         Returns a list of dictionaries with information about the known
         hosts. The dict-keys are: 'service', 'index', 'status', 'mac',
@@ -269,7 +267,7 @@ class FritzWLAN(AbstractLibraryBase):
             })
         return information
 
-    def get_info(self):
+    def get_info(self) -> dict:
         """
         Returns a dictionary with general internal information about
         the current wlan network according to the AVM documentation.
@@ -277,15 +275,15 @@ class FritzWLAN(AbstractLibraryBase):
         return self._action("GetInfo")
 
     @property
-    def is_enabled(self):
-        """Returns whether the guest network is enabled."""
+    def is_enabled(self) -> bool:
+        """Returns whether the network is enabled."""
         return self.get_info()["NewEnable"]
 
-    def enable(self):
+    def enable(self) -> None:
         """Enables the associated network."""
         self._set_enable(True)
 
-    def disable(self):
+    def disable(self) -> None:
         """Disables the associated network."""
         self._set_enable(False)
 
@@ -293,11 +291,15 @@ class FritzWLAN(AbstractLibraryBase):
         """Helper function for enable|disable."""
         self._action("SetEnable", arguments={"NewEnable": status})
 
-    def get_password(self):
+    def get_password(self) -> str:
         """Returns the current password of the associated wlan."""
         return self._action("GetSecurityKeys")["NewKeyPassphrase"]
 
-    def set_password(self, password=None, length=DEFAULT_PASSWORD_LENGTH):
+    def set_password(
+        self,
+        password: str | None = None,
+        length: int = DEFAULT_PASSWORD_LENGTH
+    ) -> None:
         """
         Sets a new password for the associated wlan.
         If no password is given a new one is created with the given
